import re

from base import baseNormalize
#from plone.i18n.normalizer.interfaces import IIDNormalizer
#from plone.i18n.normalizer.interfaces import IFileNameNormalizer
#from plone.i18n.normalizer.interfaces import IURLNormalizer

from zope.component import queryUtility
from zope.interface import implements

# Define and compile static regexes
FILENAME_REGEX = re.compile(r"^(.+)\.(\w{,4})$")
IGNORE_REGEX = re.compile(r"['\"]")
NON_WORD_REGEX = re.compile(r"[\W\-]+")
DANGEROUS_CHARS_REGEX = re.compile(r"[!#$%&()*+,/:;<=>?@\\^_{|}\[\]~`]+")
MULTIPLE_DASHES_REGEX = re.compile(r"\-+")
EXTRA_DASHES_REGEX = re.compile(r"(^\-+)|(\-+$)")
#Define static constraints
MAX_LENGTH = 50
MAX_FILENAME_LENGTH = 1023
MAX_URL_LENGTH = 255


def cropName(base, maxLength=MAX_LENGTH):
    baseLength = len(base)

    index = baseLength
    while index > maxLength:
        index = base.rfind('-', 0, index)

    if index == -1 and baseLength > maxLength:
        base = base[:maxLength]

    elif index > 0:
        base = base[:index]

    return base


class IDNormalizer(object):
    """
    This normalizer can normalize any unicode string and returns a
    version that only contains of ASCII characters allowed in a typical
    scripting or programming language id, such as CSS class names or Python
    variable names for example.

    Let's make sure that this implementation actually fulfills the API.

      >>> from zope.interface.verify import verifyClass
      >>> verifyClass(IIDNormalizer, IDNormalizer)
      True
    """
#    implements(IIDNormalizer)

    def normalize(self, text, locale=None, max_length=MAX_LENGTH):
        """
        Returns a normalized text. text has to be a unicode string and locale
        should be a normal locale, for example: 'pt_BR', 'sr@Latn' or 'de'
        """
        if locale is not None:
            # Try to get a normalizer for the locale
            util = queryUtility(IIDNormalizer, name=locale)
            parts = locale.split('_')
            if util is None and len(parts) > 1:
                # Try to get a normalizer for the base language if we asked
                # for one for a language/country combination and found none
                util = queryUtility(IIDNormalizer, name=parts[0])
            if util is not None:
                text = util.normalize(text, locale=locale)

        text = baseNormalize(text)

        # lowercase text
        base = text.lower()
        ext  = ''

        # replace whitespace and punctuation, but preserve filename extensions
        m = FILENAME_REGEX.match(text)
        if m is not None:
            base = m.groups()[0]
            ext  = m.groups()[1]

        base = IGNORE_REGEX.sub('', base)
        base = NON_WORD_REGEX.sub('-', base)
        base = MULTIPLE_DASHES_REGEX.sub('-', base)
        base = EXTRA_DASHES_REGEX.sub('', base)

        base = cropName(base, maxLength=max_length)
        
        if ext != '':
            base = base + '.' + ext

        return base


class FileNameNormalizer(object):
    """
    This normalizer can normalize any unicode string and returns a version
    that only contains of ASCII characters allowed in a file name.

    Let's make sure that this implementation actually fulfills the API.

      >>> from zope.interface.verify import verifyClass
      >>> verifyClass(IFileNameNormalizer, FileNameNormalizer)
      True
    """
#    implements(IFileNameNormalizer)

    def normalize(self, text, locale=None, max_length=MAX_FILENAME_LENGTH):
        """
        Returns a normalized text. text has to be a unicode string and locale
        should be a normal locale, for example: 'pt_BR', 'sr@Latn' or 'de'
        """
        if locale is not None:
            # Try to get a normalizer for the locale
            util = queryUtility(IFileNameNormalizer, name=locale)
            parts = locale.split('_')
            if util is None and len(parts) > 1:
                # Try to get a normalizer for the base language if we asked
                # for one for a language/country combination and found none
                util = queryUtility(IFileNameNormalizer, name=parts[0])
            if util is not None:
                text = util.normalize(text, locale=locale)

        # Preserve filename extensions
        base = text = baseNormalize(text)
        ext  = ''

        m = FILENAME_REGEX.match(text)
        if m is not None:
            base = m.groups()[0]
            ext  = m.groups()[1]

        base = IGNORE_REGEX.sub('', base)
        base = DANGEROUS_CHARS_REGEX.sub('-', base)
        base = EXTRA_DASHES_REGEX.sub('', base)
        base = MULTIPLE_DASHES_REGEX.sub('-', base)

        base = cropName(base, maxLength=max_length)

        if ext != '':
            base = base + '.' + ext

        return base


class URLNormalizer(object):
    """
    This normalizer can normalize any unicode string and returns a URL-safe
    version that only contains of ASCII characters allowed in a URL.

    Let's make sure that this implementation actually fulfills the API.

      >>> from zope.interface.verify import verifyClass
      >>> verifyClass(IURLNormalizer, URLNormalizer)
      True
    """
#    implements(IURLNormalizer)

    def normalize(self, text, locale=None, max_length=MAX_URL_LENGTH):
        """
        Returns a normalized text. text has to be a unicode string and locale
        should be a normal locale, for example: 'pt_BR', 'sr@Latn' or 'de'
        """
        if locale is not None:
            # Try to get a normalizer for the locale
            util = queryUtility(IURLNormalizer, name=locale)
            parts = locale.split('_')
            if util is None and len(parts) > 1:
                # Try to get a normalizer for the base language if we asked
                # for one for a language/country combination and found none
                util = queryUtility(IURLNormalizer, name=parts[0])
            if util is not None:
                text = util.normalize(text, locale=locale)

        text = baseNormalize(text)

        # lowercase text
        base = text.lower()
        ext  = ''

        m = FILENAME_REGEX.match(base)
        if m is not None:
            base = m.groups()[0]
            ext  = m.groups()[1]

        base = base.replace(' ', '-')
        base = IGNORE_REGEX.sub('', base)
        base = DANGEROUS_CHARS_REGEX.sub('-', base)
        base = EXTRA_DASHES_REGEX.sub('', base)
        base = MULTIPLE_DASHES_REGEX.sub('-', base)

        base = cropName(base, maxLength=max_length)

        if ext != '':
            base = base + '.' + ext

        return base

idnormalizer = IDNormalizer()
filenamenormalizer = FileNameNormalizer()
urlnormalizer = URLNormalizer()



import string
from unicodedata import normalize, decomposition

# Latin characters with accents, etc.
mapping = {
138 : 's', 140 : 'O', 142 : 'z', 154 : 's', 156 : 'o', 158 : 'z', 159 : 'Y',
192 : 'A', 193 : 'A', 194 : 'A', 195 : 'a', 196 : 'A', 197 : 'Aa', 198 : 'E',
199 : 'C', 200 : 'E', 201 : 'E', 202 : 'E', 203 : 'E', 204 : 'I', 205 : 'I',
206 : 'I', 207 : 'I', 208 : 'Th', 209 : 'N', 210 : 'O', 211 : 'O', 212 : 'O',
213 : 'O', 214 : 'O', 215 : 'x', 216 : 'O', 217 : 'U', 218 : 'U', 219 : 'U',
220 : 'U', 222 : 'th', 221 : 'Y', 223 : 's', 224 : 'a', 225 : 'a', 226 : 'a',
227 : 'a', 228 : 'ae', 229 : 'aa', 230 : 'ae', 231 : 'c', 232 : 'e', 233 : 'e',
234 : 'e', 235 : 'e', 236 : 'i', 237 : 'i', 238 : 'i', 239 : 'i', 240 : 'th',
241 : 'n', 242 : 'o', 243 : 'o', 244 : 'o', 245 : 'o', 246 : 'oe', 248 : 'oe',
249 : 'u', 250 : 'u', 251 : 'u', 252 : 'u', 253 : 'y', 254 : 'Th', 255 : 'y' }

# On OpenBSD string.whitespace has a non-standard implementation
# See http://dev.plone.org/plone/ticket/4704 for details
whitespace = ''.join([c for c in string.whitespace if ord(c) < 128])
allowed = string.ascii_letters + string.digits + string.punctuation + whitespace

def mapUnicode(text, mapping={}):
    """
    This method is used for replacement of special characters found in a mapping
    before baseNormalize is applied.
    """
    res = u''
    for ch in text:
        ordinal = ord(ch)
        if mapping.has_key(ordinal):
            # try to apply custom mappings
            res += mapping.get(ordinal)
        else:
            # else leave untouched
            res += ch
    # always apply base normalization
    return baseNormalize(res)

def baseNormalize(text):
    """
    This method is used for normalization of unicode characters to the base ASCII
    letters. Output is ASCII encoded string (or char) with only ASCII letters,
    digits, punctuation and whitespace characters. Case is preserved.

      >>> baseNormalize(123)
      '123'

      >>> baseNormalize(u'\u0fff')
      'fff'

      >>> baseNormalize(u"foo\N{LATIN CAPITAL LETTER I WITH CARON}")
      'fooI'
    """
    if not isinstance(text, basestring):
        # This most surely ends up in something the user does not expect
        # to see. But at least it does not break.
        return repr(text)

    text = text.strip()

    res = u''
    for ch in text:
        if ch in allowed:
            # ASCII chars, digits etc. stay untouched
            res += ch
        else:
            ordinal = ord(ch)
            if mapping.has_key(ordinal):
                # try to apply custom mappings
                res += mapping.get(ordinal)
            elif decomposition(ch):
                normalized = normalize('NFKD', ch).strip()
                # string may contain non-letter chars too. Remove them
                # string may result to more than one char
                res += ''.join([c for c in normalized if c in allowed])
            else:
                # hex string instead of unknown char
                res += "%x" % ordinal

    return res.encode('ascii')




import fnmatch
from zope.interface import classProvides
from zope.interface import implements
from collective.transmogrifier.interfaces import ISectionBlueprint
from collective.transmogrifier.interfaces import ISection
from collective.transmogrifier.utils import Matcher
from urllib import unquote
import urlparse
import re
import logging
logger = logging.getLogger('Plone')
from transmogrify.pathsorter.treeserializer import TreeSerializer

"""
Backlinks Title
===============

This blueprint will take the _backlinks from the item generated by webcrawler
and if not Title field has been given to the item it will attempt to guess
it from the link names that linked to this document.
You can specify an option 'ignore' option to specify titles never to use

If it can't guess it from the backlinks it will default to using the file name after
cleaning it up somewhat
"""

class BacklinksTitle(object):
    classProvides(ISectionBlueprint)
    implements(ISection)

    def __init__(self, transmogrifier, name, options, previous):
        self.previous = previous
        self.toignore=options.get('ignore','next\nprevios\n').strip().split('\n')
        self.treeserializer = TreeSerializer(transmogrifier, name, options, previous)

    def __iter__(self):
        items = []
        defaultpages  = {}
        for item in self.treeserializer:
            path = item.get('_path')
            backlinks = item.get('_backlinks')
            title = item.get('title')
            defaultpage = item.get('_defaultpage')
            if title:
                items.append( item )
                continue
            elif defaultpage:
                # save and we'll use that for title
                indexpath = urlparse.urljoin(path+'/', defaultpage)
                defaultpages[indexpath] = item
                items.append( item )
                continue
            elif not backlinks:
                self.titlefromid(item)
                items.append( item )
                continue
            names = [name for url, name in backlinks if not self.ignore(name)]
            # do a vote
            votes = {}
            for name in names:
                votes[name] = votes.get(name,0) + 1
            votes = [(c,name) for name,c in votes.items()]
            votes.sort()
            if votes:
                c,item['title'] = votes[-1]
                msg = "backlinkstitle %s (%s)" % (path,item['title'])
                logger.log(logging.DEBUG, msg)
            else:
                self.titlefromid(item)
            # go back and title the folder if this is a default page
                
            items.append( item )
        items2 = []
        for item in items:
            path = item.get('_path')
            folder = defaultpages.get(path)
            if folder:
                if 'title' in item:
                    folder['title'] = item['title']
                if 'description' in item:
                    folder['description'] = item['description']
            items2.append( item )
        for item in items2:
            yield item

    def ignore(self, name):
        for pat in self.toignore:
            if re.search(pat,name):
                return True
        return False

    def titlefromid(self, item):
        path = item.get('_path')
        if not path:
            return
        title = [p for p in path.split('/') if p][-1]
        title = unquote(title)
        title = title.split('.')[0]
        item['title'] = title
        msg = "backlinkstitle %s (%s)" % (path,item['title'])
        logger.log(logging.DEBUG, msg)




from zope.interface import implements
from zope.interface import classProvides
from zope.component import queryUtility
from plone.i18n.normalizer.interfaces import IURLNormalizer

from collective.transmogrifier.interfaces import ISectionBlueprint
from collective.transmogrifier.interfaces import ISection
import urllib
from lxml import etree
import lxml
from urlparse import urljoin
from external.relative_url import relative_url
from sys import stderr
from collective.transmogrifier.utils import Expression
import logging
from external.normalize import urlnormalizer as normalizer
import urlparse
logger = logging.getLogger('Plone')
from sys import stderr
#from plone.i18n.normalizer import urlnormalizer as normalizer

INVALID_IDS = ['security']


class Relinker(object):
    classProvides(ISectionBlueprint)
    implements(ISection)
    
    def __init__(self, transmogrifier, name, options, previous):
        self.previous = previous
        self.locale = getattr(options, 'locale', 'en')
        self.link_expr = None
        if options.get('link_expr', None):
            self.link_expr = Expression(
                    options['link_expr'],
                    transmogrifier, name, options)
        #util = queryUtility(IURLNormalizer)
        #if util:
        #    self.normalize = util.normalize
        #else:
        self.locale = Expression(options.get('locale', 'python:None'), 
                                transmogrifier, name, options)
        
    
    
    def __iter__(self):
        
        #TODO: needs to take input as to what new name should be so other blueprints can decide
        #TODO: needs allow complete changes to path so can move structure
        #TODO: needs to change file extentions of converted docs. or allow others to change that
        #TODO need to fix relative links 
        

        changes = {}
        bad = {}
        for item in self.previous:
            path = item.get('_path',None)
            if not path:
                url = item.get('_bad_url')
                if url:
                    bad[url] = item
                yield item
                continue
            base = item.get('_site_url','')
            
            def norm(part):
                #TODO - don't normalize to existing names
                if part[0] == '_':
                    part = part[1:]+'-1'
                # Get the information we require for normalization
                keywords = dict(text=urllib.unquote_plus(part), locale=self.locale(item))
                # Perform Normalization
                part = normalizer.normalize(**keywords)
                if part in INVALID_IDS:
                    return part+'-1'
                else:
                    return part 
            newpath = '/'.join([norm(part) for part in path.split('/')])
            origin = item.get('_origin')
            if not origin:
                origin = item['_origin'] = path
            item['_path'] = newpath
            #normalize link
            link = urllib.unquote_plus(base+origin)
            #assert not changes.get(link,None), str((item,changes.get(base+origin,None)))
                
            changes[link] = item

        for item in changes.values():
            if 'text' in item and item.get('_mimetype') in ['text/xhtml', 'text/html']:
                relinkHTML(item, changes, bad, link_expr=self.link_expr)
            del item['_origin']
            #rewrite the backlinks too
            backlinks = item.get('_backlinks',[])
            newbacklinks = []
            for origin,name in backlinks:
                #assume absolute urls
                backlinked= changes.get(origin)
                if backlinked:
                    backlink = backlinked['_site_url']+backlinked['_path']
                    newbacklinks.append((backlink,name))
                else:
                    newbacklinks.append((origin,name))
            if backlinks:
                item['_backlinks'] = newbacklinks
    
            yield item

def relinkHTML(item, changes, bad={}, link_expr=None):        
    path = item['_path']
    oldbase = item['_site_url']+item['_origin']
    newbase = item['_site_url']+path
    def swapfragment(link, newfragment):
        t = urlparse.urlparse(link)
        fragment = t[-1]
        t = t[:-1] + (newfragment,)
        link = urlparse.urlunparse(t)
        return link, fragment
    
    def replace(link):
        link, fragment = swapfragment(link, '')

        linked = changes.get(link)
        if not linked:
            link = urllib.unquote_plus(link)
            linked = changes.get(link)
            
        if linked:
            if link_expr:
                linkedurl = item['_site_url']+link_expr(linked)
            else:
                linkedurl = item['_site_url']+linked['_path']
            return swapfragment(relative_url(newbase, linkedurl), fragment)[0]
        else:
            #if path.count('commercial-part-codes.doc'):
            if link not in bad:
                msg = "relinker: no match for %s in %s" % (link,path)
                logger.log(logging.DEBUG, msg)
                print >> stderr, msg
            return swapfragment(relative_url(newbase, link), fragment)[0]
    
    tree = lxml.html.fragment_fromstring(item['text'])
    tree.rewrite_links(replace, base_href=oldbase)
    item['text'] = etree.tostring(tree,pretty_print=True,encoding=unicode)
 #   except Exception:
 #       msg = "ERROR: relinker parse error %s, %s" % (path,str(Exception))
 #       logger.log(logging.ERROR, msg, exc_info=True)



#!/usr/bin/env python
# -*- coding: iso-8859-1 -*-
#
# Create a relative URL given to absolute URLs
# See tests at the end:
# test_relative_url("http://foo/a/b", "http://foo/c", "../c")
#
# (c) May 2002 Thomas Guettler http://www.thomas-guettler.de
# This code is in the public domain
# Feedback Welcome
#

import urlparse
import re
import string


def relative_url(source, target):
    su=urlparse.urlparse(source)
    tu=urlparse.urlparse(target)
    junk=tu[3:]
    if su[0]!=tu[0] or su[1]!=tu[1]:
        #scheme (http) or netloc (www.heise.de) are different
        #return absolut path of target
        return target
    su=re.split("/", su[2])
    tu=re.split("/", tu[2])
    su.reverse()
    tu.reverse()

    #remove parts which are equal   (['a', 'b'] ['a', 'c'] --> ['c'])
    while len(su)>0 and len(tu)>0 and su[-1]==tu[-1]:
        su.pop()
        last_pop=tu.pop()
    if len(su)==0 and len(tu)==0:
        #Special case: link to itself (http://foo/a http://foo/a -> a)
        tu.append(last_pop)
    if len(su)==1 and su[0]=="" and len(tu)==0:
        #Special case: (http://foo/a/ http://foo/a -> ../a)
        su.append(last_pop)
        tu.append(last_pop)
    tu.reverse()
    relative_url=[]
    for i in range(len(su)-1): 
        relative_url.append("..")
    rel_url=string.join(relative_url + tu, "/")
    rel_url=urlparse.urlunparse(["", "", rel_url, junk[0], junk[1], junk[2]])
    return rel_url

def test_relative_url(source, target, result):
    res=relative_url(source, target)
    if res!=result:
        print "Test FAILED: result is:", res, "should be:", result, \
              "source:", source, "target:", target
    else:
        print "Test ok: result is:", res, "source:", source, "target:", target
        
if __name__ == "__main__":
    test_relative_url("http://foo/a/b", "http://foo/c", "../c")
    test_relative_url("http://foo/a/b", "http://foo/c/d", "../c/d")
    test_relative_url("http://foo/a/b", "ftp://foo/c", "ftp://foo/c")
    test_relative_url("http://foo/a", "http://foo/b", "b")
    test_relative_url("http://foo/a/", "http://foo/b", "../b")
    test_relative_url("http://foo:80/a/", "http://foo/b", "http://foo/b")
    test_relative_url("http://foo:8080/a/", "http://foo/b", "http://foo/b")
    test_relative_url("http://foo/a", "http://foo/a", "a")
    test_relative_url("http://foo/a/", "http://foo/a", "../a")
    test_relative_url("http://foo/a", "http://foo/b/c", "b/c")
    test_relative_url("http://foo/a/", "http://foo/b/c", "../b/c")
    test_relative_url("http://foo/a/b", "http://foo/c/d", "../c/d")
    test_relative_url("http://foo/a/b/", "http://foo/c/d", "../../c/d")
    test_relative_url("http://foo/a", "http://foo/b", "b")
    test_relative_url("http://foo/a;para?query#frag", "http://foo/a", "a")
    test_relative_url("http://foo/a", "http://foo/a;para?query#frag",
                      "a;para?query#frag")
    test_relative_url("a/b", "a/c", "c")




import unittest

from zope.testing import doctest
from zope.component import provideUtility
from Products.Five import zcml
from zope.component import provideUtility
from zope.interface import classProvides, implements
from collective.transmogrifier.interfaces import ISectionBlueprint, ISection

from Testing import ZopeTestCase as ztc
from Products.PloneTestCase import PloneTestCase as ptc
from Products.PloneTestCase.layer import onsetup
from Products.Five import zcml
from Products.Five import fiveconfigure

from collective.transmogrifier.tests import setUp as baseSetUp
from collective.transmogrifier.tests import tearDown
from collective.transmogrifier.sections.tests import PrettyPrinter
from collective.transmogrifier.sections.tests import SampleSource

from transmogrify.webcrawler.webcrawler import WebCrawler
from transmogrify.webcrawler.treeserializer import TreeSerializer
from transmogrify.webcrawler.typerecognitor import TypeRecognitor
from transmogrify.webcrawler.safeportaltransforms import  SafePortalTransforms
from transmogrify.webcrawler.makeattachments import MakeAttachments
from templatefinder import TemplateFinder
from transmogrify.webcrawler.relinker import Relinker
from transmogrify.webcrawler.simplexpath import SimpleXPath
from plone.i18n.normalizer import urlnormalizer
from lxml import etree
import lxml.html
import lxml.html.soupparser
from lxml.html.clean import Cleaner
import urlparse
import transmogrify.webcrawler
from os.path import dirname, abspath
import urllib


class HTMLSource(object):
    classProvides(ISectionBlueprint)
    implements(ISection)

    def __init__(self, transmogrifier, name, options, previous):
        self.previous = previous
        self.items = []
        for order,item in zip(range(0,len(options)),options.items()):
            path,text = item 
            if path in ['blueprint']:
                continue
            item_ = dict(_mimetype="text/html",
                        _site_url="http://test.com/",
                        _path=path,    
                        text=text,
                        _sortorder=order)
            self.items.append(item_)

    def __iter__(self):
        for item in self.previous:
            yield item

        for item in self.items:
            yield item

class HTMLBacklinkSource(HTMLSource):
    classProvides(ISectionBlueprint)
    implements(ISection)

    def __init__(self, transmogrifier, name, options, previous):
        HTMLSource.__init__(self, transmogrifier, name, options, previous)
        pathtoitem = {}
        for item in self.items:
            pathtoitem[item['_site_url']+item['_path']] = item
        for item in self.items:
            parser = lxml.html.soupparser.fromstring(item['text'])
            for element, attribute, rawlink, pos in parser.iterlinks():
                t = urlparse.urlparse(rawlink)
                fragment = t[-1]
                t = t[:-1] + ('',)
                rawlink = urlparse.urlunparse(t)
                base = item['_site_url']+item['_path']
                link = urlparse.urljoin(base, rawlink)
                linked = pathtoitem.get(link)
                if linked:
                    linked.setdefault('_backlinks',[]).append((base,element.text_content()))


class MockPortalTransforms(object):
    def __call__(self, transform, data):
        return 'Transformed %i using the %s transform' % (len(data), transform)
    def convertToData(self, target, data, mimetype=None):
        html='<img src="image01.jpg"><img src="image02.jpg">'
        class dummyfile:
            def __init__(self, text):
                self.text = text
            def __str__(self):
                return self.text+html
            def getSubObjects(self):
                return {'image01.jpg':data,'image02.jpg':data}
        if mimetype is not None:
            return dummyfile( 'Transformed %i from %s to %s' % (
                len(data), mimetype, target) )
        else:
            return dummyfile('Transformed %r to %s' % (data, target) )
    def convertTo(self, target, data, mimetype=None):
        return self.convertToData(target,data,mimetype)



def setUp(test):
    baseSetUp(test)

    from collective.transmogrifier.transmogrifier import Transmogrifier
    test.globs['transmogrifier'] = Transmogrifier(test.globs['plone'])

    import zope.component
    import collective.transmogrifier.sections
    zcml.load_config('meta.zcml', zope.app.component)
    zcml.load_config('configure.zcml', collective.transmogrifier.sections)

    test.globs['plone'].portal_transforms = MockPortalTransforms()

    provideUtility(PrettyPrinter,
        name=u'collective.transmogrifier.sections.tests.pprinter')
    provideUtility(WebCrawler,
        name=u'transmogrify.webcrawler.webcrawler')
    provideUtility(TreeSerializer,
        name=u'transmogrify.webcrawler.treeserializer')
    provideUtility(TypeRecognitor,
        name=u'transmogrify.webcrawler.typerecognitor')
    provideUtility(TemplateFinder,
        name=u'transmogrify.webcrawler.templatefinder')
    provideUtility(urlnormalizer)
    provideUtility(Relinker,
        name=u'transmogrify.webcrawler.relinker')
    provideUtility(SimpleXPath,
        name=u'transmogrify.webcrawler.simplexpath')
    provideUtility(SafePortalTransforms,
        name=u'transmogrify.webcrawler.safeportaltransforms')
    from backlinkstitle import BacklinksTitle
    provideUtility(BacklinksTitle,
        name=u'transmogrify.webcrawler.backlinkstitle')
    from isindex import IsIndex
    provideUtility(IsIndex,
        name=u'transmogrify.webcrawler.isindex')
    from pathmover import PathMover
    provideUtility(PathMover,
        name=u'transmogrify.webcrawler.pathmover')
    from safeatschemaupdater import SafeATSchemaUpdaterSection
    provideUtility(SafeATSchemaUpdaterSection,
        name=u'transmogrify.webcrawler.safeatschemaupdater')
    from constructor import SafeConstructorSection
    provideUtility(SafeConstructorSection,
        name=u'transmogrify.webcrawler.constructor')
    from makeattachments import MakeAttachments
    provideUtility(MakeAttachments,
        name=u'transmogrify.webcrawler.makeattachments')
    from debugsection import DebugSection
    provideUtility(DebugSection,
        name=u'transmogrify.webcrawler.debugsection')
    from staticcreator import StaticCreatorSection
    provideUtility(StaticCreatorSection,
        name=u'transmogrify.webcrawler.staticcreator')

    provideUtility(HTMLSource,
        name=u'transmogrify.webcrawler.test.htmlsource')
    provideUtility(HTMLBacklinkSource,
        name=u'transmogrify.webcrawler.test.htmlbacklinksource')


def SafeATSchemaUpdaterSetUp(test):
    setUp(test)

    from Products.Archetypes.interfaces import IBaseObject
    class MockPortal(object):
        implements(IBaseObject)

        def unrestrictedTraverse(self, path, default):
            return self

        _file_value = None
        _file_filename = None
        _file_mimetype = None
        _file_field = None

        def set(self, name, value, **arguments):
            self._file_field = name
            self._file_value = value
            if 'mimetype' in arguments:
                self._file_mimetype = arguments['mimetype']
            if 'filename' in arguments:
                self._file_filename = arguments['filename']

        def get(self, name):
            return self._file_value

        def checkCreationFlag(self):
            pass

        def unmarkCreationFlag(self):
            pass

        def getField(self, name):
            return self

    test.globs['plone'] = MockPortal()
    test.globs['transmogrifier'].context = test.globs['plone']

    class SafeATSchemaUpdaterSectionSource(SampleSource):
        classProvides(ISectionBlueprint)
        implements(ISection)

        def __init__(self, *args, **kw):
            super(SafeATSchemaUpdaterSectionSource, self).__init__(*args, **kw)
            self.sample = (
                {'_path': '/dummy',
                 'file': 'image content',
                 'file.filename': 'image.jpg',
                 'file.mimetype': 'image/jpeg',},
            )
    provideUtility(SafeATSchemaUpdaterSectionSource,
        name=u'transmogrify.webcrawler.tests.safeatschemaupdatersource')

def MakeAttachmentsSetUp(test):
    setUp(test)

    class MakeAttachmentsSource(SampleSource):
        classProvides(ISectionBlueprint)
        implements(ISection)

        def __init__(self, *args, **kw):
            super(MakeAttachmentsSource, self).__init__(*args, **kw)
            self.sample = (
                {'_site_url': 'http://www.test.com',
                 '_path': '/item1',},
                {'_site_url': 'http://www.test.com',
                 '_path': '/subitem1',
                 '_backlinks': [('http://www.test.com/subitem2', '')],
                 'title': 'subitem1 title',
                 'decription': 'test if condition is working',
                 '_type': 'Document'},
                {'_site_url': 'http://www.test.com',
                 '_path': '/subitem2',
                 '_backlinks': [('http://www.test.com/subitem1', '')],
                 'title': 'subitem2 title',
                 'image': 'subitem2 image content',
                 '_type': 'Image'},
            )
    provideUtility(MakeAttachmentsSource,
        name=u'transmogrify.webcrawler.tests.makeattachments')
    provideUtility(MakeAttachments,
        name=u'transmogrify.webcrawler.makeattachments')

@onsetup
def setup_product():
    """ """
    fiveconfigure.debug_mode = True
    zcml.load_config('configure.zcml', transmogrify.webcrawler)
    fiveconfigure.debug_mode = False
    ztc.installPackage('plone.app.z3cform')
#    ztc.installPackage('lovely.remotetask')
    ztc.installPackage('transmogrify.webcrawler')


setup_product()
#ptc.setupPloneSite(extension_profiles=('transmogrify.webcrawler:default',), with_default_memberarea=False)
ptc.setupPloneSite(products=['transmogrify.webcrawler'])

class TestCase(ptc.FunctionalTestCase):
    """ We use this base class for all the tests in this package. If necessary,
        we can put common utility or setup code in here. This applies to unit
        test cases. """
    _configure_portal = False

    def beforeTearDown(self):
        pass

    def afterSetUp(self):
        here = abspath(dirname(__file__))
        url = urllib.pathname2url(here)
        self.testsite = 'file://%s/test_staticsite' % url

        self.portal.error_log._ignored_exceptions = ()

        self.portal.acl_users.portal_role_manager.updateRolesList()

        self.portal.acl_users._doAddUser('manager', 'pass', ('Manager',), [])
        self.login('manager')



        from Products.Five.testbrowser import Browser
        self.browser = Browser()
#        self.setRoles(('Manager',))
        self.browser.open(self.portal.absolute_url()+'/login_form')
        self.browser.getControl(name='__ac_name').value = 'manager'
        self.browser.getControl(name='__ac_password').value = 'pass'
        self.browser.getControl(name='submit').click()
        self.browser.open(self.portal.absolute_url())


def test_suite():
    flags = optionflags = doctest.ELLIPSIS | doctest.REPORT_ONLY_FIRST_FAILURE | \
                        doctest.NORMALIZE_WHITESPACE | doctest.REPORT_UDIFF

    return unittest.TestSuite((
        doctest.DocFileSuite('webcrawler.txt', 
                setUp=setUp, 
                optionflags = flags,
                tearDown=tearDown),

#        doctest.DocFileSuite('treeserializer.txt', 
#                setUp=setUp, 
#                tearDown=tearDown, 
#                optionflags=flags),
#        doctest.DocFileSuite('typerecognitor.txt', 
#                setUp=setUp, 
#                optionflags = flags,
#                tearDown=tearDown),
#        doctest.DocFileSuite('templatefinder.txt', 
#                setUp=setUp, 
#                optionflags = flags,
#                tearDown=tearDown),
#        doctest.DocFileSuite('relinker.txt', 
#                setUp=setUp, 
#                optionflags = flags,
#                tearDown=tearDown),
#        doctest.DocFileSuite('pathmover.txt', 
#                setUp=setUp, 
#                optionflags = flags,
#                tearDown=tearDown),
#        doctest.DocFileSuite('simplexpath.txt', 
#                setUp=setUp, 
#                optionflags = flags,
#                tearDown=tearDown),
#        doctest.DocFileSuite('testsites.txt', 
#                setUp=setUp,
#                optionflags = flags,
#                tearDown=tearDown),
#        doctest.DocFileSuite('safeatschemaupdater.txt',
#                setUp=SafeATSchemaUpdaterSetUp,
#                optionflags = flags,
#                tearDown=tearDown),
#        doctest.DocFileSuite('makeattachments.txt',
#                setUp=MakeAttachmentsSetUp,
#                optionflags = flags,
#                tearDown=tearDown),
#        doctest.DocFileSuite('isindex.txt',
#                setUp=MakeAttachmentsSetUp,
#                optionflags = flags,
#                tearDown=tearDown),
#        doctest.DocFileSuite('safeportaltransforms.txt',
#                setUp=MakeAttachmentsSetUp,
#                optionflags = flags,
#                tearDown=tearDown),
#        ztc.FunctionalDocFileSuite(
#            'README.txt',
#             package='transmogrify.webcrawler',
#             test_class=TestCase,
##            tearDown=zc.buildout.testing.buildoutTearDown,
#             optionflags = flags,
#            #globs=globs,
##            checker=renormalizing.RENormalizing([
##               zc.buildout.testing.normalize_path,
#               #zc.buildout.testing.normalize_script,
#               #zc.buildout.testing.normalize_egg_py,
#               #zc.buildout.tests.normalize_bang,
# #              ]),
#            ),



    ))

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')






from zope.interface import classProvides
from zope.interface import implements
from collective.transmogrifier.interfaces import ISectionBlueprint
from collective.transmogrifier.interfaces import ISection
from collective.transmogrifier.utils import Condition, Expression
import logging
logger = logging.getLogger('Plone')



class MakeAttachments(object):
    classProvides(ISectionBlueprint)
    implements(ISection)

    def __init__(self, transmogrifier, name, options, previous):
        self.previous = previous
        self.fields=Expression(options.get('fields','python:False'), transmogrifier, name, options)
        self.condition=Condition(options.get('condition','python:True'), transmogrifier, name, options)
        self.defaultpage = options.get('defaultpage','index-html')

    def __iter__(self):

        # split items on subitems and other
        items, subitems = [], {}
        for item in self.previous:
            backlinks = item.get('_backlinks',[])
            if len(backlinks) == 1:
                link,name = backlinks[0]
                subitems.setdefault(link, [])
                subitems[link].append(item)
            items.append(item)
 
        # apply new fields from subitems to items 
        total = 0
        merged = 0
        for item in items:
            base = item.get('_site_url',None)
            origin = item.get('_origin',item.get('_path',None))
            if not base or not origin:
                yield item
                continue
            links = subitems.get(base+origin,[])
            backlinks =  item.get('_backlinks',[])
            if not links and len(backlinks)==1 and subitems.get(backlinks[0][0]) is not None:
                continue #item is a deadend and will be delt with elsewhere
            folder=None
            i = 0
            attach = []
            for subitem in links:
                subbase = subitem.get('_site_url',None)
                suborigin = subitem.get('_origin',subitem.get('_path',None))
                    
                if subitems.get(subbase+suborigin,[]):
                    # subitem isn;t a deadend and will be dealt with elsewhere. 
                    continue
                if not self.condition(item,i=i,subitem=subitem):
                    yield subitem
                    continue
                change = self.fields(item, subitem=subitem, i=i)
                if change:
                    item.update(dict(change))
                    msg = "imakeattachments: %s to %s{%s}" %(suborigin,origin,dict(change).keys())
                    logger.log(logging.DEBUG, msg)
                    # now pass a move request to relinker
                    file,text=change[0]
                    attach.append(dict(_origin=suborigin,
                               _path=file,
                               _site_url=subbase))
                else: #turn into default folder
                    if not folder:
                        folder = dict(_path=item['_path'],
                                      _site_url=base,
                                      _type="Folder",
                                      _defaultpage=self.defaultpage)
                        if not item.get('_origin'):
                            item['_origin']=item['_path']
                        item['_path'] = '/'.join(item['_path'].split('/') + [self.defaultpage])
                    if '_origin' not in subitem:
                        subitem['_origin'] = subitem['_path']
                    file = subitem['_path'].split('/')[-1]
                    subitem['_path'] = '/'.join(folder['_path'].split('/') + [file])
                    yield subitem
                i = i +1
            if folder:
                yield folder
            yield item
            # got to set actual final paths of attachments moves
            for subitem in attach:
                subitem['_path'] = '/'.join(item['_path'].split('/')+[subitem['_path']])
                yield subitem
                
        

        





import fnmatch
from zope.interface import classProvides
from zope.interface import implements
from collective.transmogrifier.interfaces import ISectionBlueprint
from collective.transmogrifier.interfaces import ISection
from collective.transmogrifier.utils import Matcher

from lxml import etree
import lxml.html
import lxml.html.soupparser

from StringIO import StringIO
from sys import stderr
import logging
logger = logging.getLogger('Plone')



class IsIndex(object):
    classProvides(ISectionBlueprint)
    implements(ISection)



    def __init__(self, transmogrifier, name, options, previous):
        self.previous = previous
        self.min_links=options.get('min_links',2)
        self.max_uplinks=options.get('max_uplinks',2)
            

    def __iter__(self):
      
        #Stri
      
                    
        self.moved= {}
        items = {}
        ulinks = {}
        for item in self.previous:
            path,html = self.ishtml(item)
            if not path:
                yield item
                continue
            
            tree = lxml.html.fragment_fromstring(html)
            base = item.get('_site_url','')
            tree.make_links_absolute(base+path)
            if '_origin' in item:
                self.moved[item['_origin']] = path
            links = []
            items[path] = (item,path,links)
            for element, attribute, link, pos in tree.iterlinks():
                if attribute == 'href' and link not in ulinks:
                    ulinks[link] = True
                    if link.startswith(base):
                        link = link[len(base):]
                    link = '/'.join([p for p in link.split('/') if p])
                    links.append(link)
        done = []
        while items:
            indexes = {}
            for item,path,links in items.values():
                if not links:
                    continue
                count, dir, rest = self.indexof(links)
                print >>stderr, (count,len(links),dir,path,item.get('_template'), rest)
                if self.isindex(count,links):
                    indexes.setdefault(dir,[]).append((count,item,path,links,dir))
    
            mostdeep = [(len(dir.split('/')),i) for dir,i in indexes.items()]
            if not mostdeep:
                break
            mostdeep.sort()
            depth,winner = mostdeep[-1]
            self.move(winner)
            for count,item,path,links,dir in winner:
                del items[path]
                yield item
        for item,path,links in items.values():
            yield item
                    
                
                     
    def move(self, items):
        if not items: return
        items.sort()
        count,item,toppath,links,dir = items[-1]
        for count,item,path,links,dir in items:
            if False: #path == toppath: #TODO need a better way to work out default view
                file = 'index_html'
            else:
                file = path.split('/')[-1]
            
            if dir:
                target = dir+'/'+file
            else:
                target = file
            if item['_path'] == target:
                continue
            item.setdefault('_origin',path)
            item['_path'] = target
                
            self.moved[path] = item['_path']
            msg = "isindex: moved %s to %s" %(path,dir+'/'+file)
            logger.log(logging.DEBUG, msg)

    def isindex(self, count, links):
        return count >= self.min_links and count>=len(links)-self.max_uplinks
            

    def indexof(self, links):
        dirs = {}
        def most(count):
            return self.isindex(count,links)

        for link in links:
            newlink = self.moved.get(link)
            if newlink:
                link = newlink
                
            dir = '/'.join([p for p in link.split('/') if p][:-1])
            dirs[dir] = dirs.get(dir,0) + 1
        if not dirs:
            return 0,None
        alldirs = dirs
        while True:
            tops = [(count,dir) for dir,count in dirs.items()]
            tops.sort()
            count,dir = tops[-1]
            if most(count) or len(tops)<2:
                break
            #find common dir. Take longest and make it shorter
            common = [(dir,count) for count,dir in tops]
            common.sort()
            common.reverse()
            longdir,longcount = common[0]
            longdir = '/'.join(longdir.split('/')[:-1])
            dirs = dict(common[1:])
            dirs[longdir] = dirs.get(longdir,0) + longcount
            
        return count,dir, alldirs
    
#            tops = []
#            found = False
#            for dir,count in common[1:]:
#                if not found and longdir.startswith(dir) and dir:
#                    count = count+longcount
#                    found = True
#                tops.append((count,dir))


    def ishtml(self, item):
              path = item.get('_path',None)
              content = item.get('_content',None) or item.get('text',None)
              mimetype = item.get('_mimetype',None)
              if  path and content and mimetype in ['text/xhtml', 'text/html']:
                  return path,content
              else:
                  return None,None
       



from setuptools import setup, find_packages
import os

version = '1.0a1'

setup(name='transmogrify.siteanalyser',
      version=version,
      description="transmogrifier source blueprints for crawling html",
      long_description=open('README.txt').read() +'\n'+
                       open(os.path.join("transmogrify", "siteanalyser", "isindex.txt")).read() + "\n" +
                       open(os.path.join("transmogrify", "siteanalyser", "relinker.txt")).read() + "\n" +
                       open(os.path.join("transmogrify", "siteanalyser", "makeattachments.txt")).read() + "\n" +
                       #open(os.path.join("transmogrify", "siteanalyser", "backlinkstitle.txt")).read() + "\n" +
                       open(os.path.join("docs", "HISTORY.txt")).read(),
      # Get more strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='transmogrifier blueprint funnelweb source plone import conversion microsoft office',
      author='Dylan Jay',
      author_email='software@pretaweb.com',
      url='http://github.com/djay/transmogrify.siteanalyser',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['transmogrify'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          # -*- Extra requirements: -*-
          'lxml',
          'BeautifulSoup',
          'collective.transmogrifier',
          'plone.i18n'
          ],
      entry_points="""
            [z3c.autoinclude.plugin]
            target = transmogrify
            """,
            )
